package com.liang.mapper;

import com.liang.dao.IUserDao;
import com.liang.domain.User;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.cache.CacheManager;
import org.springframework.util.Assert;

import java.util.List;

/**
 * @PROJECT_NAME: springboot-leaning
 * @USER: yuliang
 * @DESCRIPTION:
 * @DATE: 2021-06-30 16:41
 */

@SpringBootTest
public class UserDaoTest {

    private static final String CACHE_NAME_USER = "users";


    @Autowired
    private IUserDao userDao;

    @Autowired
    private CacheManager cacheManager;

    @Test
    public void testCacheManager() {
        System.out.println(cacheManager);
    }

    @Test
    public void testGetTotalCount(){
        int totalCount = userDao.getTotalCount();
        System.out.println(totalCount);
    }

    @Test
    public void testGetUserById() {
        // 这里，事先插入一条 id = 1 的记录。
        Integer id = 1;

        // 查询 id = 1 的记录,从数据库查询会打印SQL
        User user = userDao.getUser(id);
        System.out.println("第一次查询，user：" + user);
        // 判断缓存中，是不是存在
        Assert.notNull(cacheManager.getCache(CACHE_NAME_USER).get(user.getId(), User.class), "缓存为空");

        // 查询 id = 1 的记录,从缓冲获取，不打印SQL
        user = userDao.getUser(id);
        System.out.println("第二次查询，user：" + user);
    }

    @Test
    public void testInsertUser(){

        User user = new User();
        user.setAge(18);
        user.setMobile("13601482796");
        user.setUsername("xiaoliang");

        userDao.insertUser(user);
    }

    @Test
    public void testGetUsers(){

        List<User> users = userDao.getUsers();
        System.out.println(users);

    }

    @Test
    public void testUpdateUserNameById(){

        Integer id = 7;
        userDao.updateUserNameById(id, "test");

    }

    @Test
    public void testDeleteUser(){

        Integer id = 1;

        userDao.deleteUser(id);

        // 查询 id = 1 的记录
        User user = userDao.getUser(id);

        System.out.println("查询，user：" + user);
    }

    @Test
    public void testUpdateUser(){

        User user = new User();
        user.setId(2);
        user.setAge(10);
        user.setUsername("yuliang");
        user.setMobile("12310046789");

        userDao.updateUser(user);
    }


}
